@@ -1,7 +1,7 @@
 #
 #       images.R
 #
-#      $Revision: 1.171 $     $Date: 2022/03/24 03:22:07 $
+#      $Revision: 1.178 $     $Date: 2023/05/04 00:58:05 $
 #
 #      The class "im" of raster images
 #
@@ -311,7 +311,7 @@ shift.im <- function(X, vec=c(0,0), ..., origin=NULL) {
         if(jtype == "given")
           warning("Argument j ignored")
         W <- raster %orifnull% as.owin(x)
-        M <- as.mask.psp(as.psp(i), W=W, ...)
+        M <- psp2mask(as.psp(i), W=W, ...)
         xM <- x[M, drop=drop]
         if(is.im(xM)) xM <- spatstat.linnet::linim(i, xM)
         return(xM)
@@ -491,7 +491,8 @@ update.im <- function(object, ...) {
       } else {
         vnew[!is.na(v)] <- as.integer(value)
       }
-      v <- factor(vnew, labels=levels(value))
+      lnew <- levels(value)
+      v <- factor(vnew, levels=seq_along(lnew), labels=lnew)
     }
     X$v <- v
     return(update(X))
@@ -1079,12 +1080,23 @@ quantile.im <- function(x, ...) {
   return(q)
 }
 
+quantilefun.im <- function(x, ..., type=1) {
+  verifyclass(x, "im")
+  f <- ecdf(as.numeric(x[]))
+  quantilefun(f, type=type)
+}
+  
 integral <- function(f, domain=NULL, ...) {
   UseMethod("integral")
 }
 
-integral.im <- function(f, domain=NULL, ...) {
+integral.im <- function(f, domain=NULL, weight=NULL, ...) {
   verifyclass(f, "im")
+  if(!is.null(weight)) {
+    if(is.function(weight))
+      weight <- as.im(weight, W=as.owin(f))
+    f <- f * weight
+  }
   typ <- f$type
   if(!any(typ == c("integer", "real", "complex", "logical")))
     stop(paste("Don't know how to integrate an image of type", sQuote(typ)))
